#
# Copyright (c) 2024, Neptune Labs Sp. z o.o.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
__all__ = ["StructureVersion"]

from enum import Enum


class StructureVersion(Enum):
    # -------------------------------------------------
    # .neptune/
    #     async/
    #         <uuid>/
    #            exec-<num><timestamp>/
    #               container_type
    #               data-1.log
    #               ...
    # -------------------------------------------------
    LEGACY = 1

    # -------------------------------------------------
    # .neptune/
    #     async/
    #         run__<uuid>/
    #            exec-<timestamp>-<date>-<pid>/
    #               data-1.log
    #               ...
    # -------------------------------------------------
    CHILD_EXECUTION_DIRECTORIES = 2

    # -------------------------------------------------
    # .neptune/
    #     async/
    #         run__<uuid>__<pid>__<random_key>/
    #             data-1.log
    #             ...
    # -------------------------------------------------
    DIRECT_DIRECTORY = 3
